"ash1"<- function(bins, m = 5, kopt = c(2, 2), warn = T)
{
	nc <- bins$nc
	ab <- bins$ab
	nbin <- length(nc)
	r <- .C("ash1",
		as.integer(m),
		as.integer(nc),
		as.integer(nbin),
		as.single(ab),
		as.integer(kopt),
		t = single(nbin),
		f = single(nbin),
		single(m),
		ier = integer(1))
	if(warn & r$ier == 1)
		print("ash estimate nonzero outside interval ab")
	list(x = r$t, y = r$f, m = m, ab = ab, kopt = kopt, ier = r$ier)
}

"ash2"<- function(bins, m = c(5, 5), kopt = c(2, 2), warn = T)
{
	if(!is.matrix(bins$nc))
		stop("bins does not contain bin count matrix")
	if(!is.matrix(bins$ab))
		stop("ab not a matrix - should be 2 by 2")
	nc <- bins$nc
	ab <- bins$ab
	nbin <- dim(nc)
	r <- .C("ash2",
		as.integer(m),
		as.integer(nc),
		as.integer(nbin[1]),
		as.integer(nbin[2]),
		as.single(ab),
		as.integer(kopt),
		f = single(nbin[1] * nbin[2]),
		single(m[1] * m[2]),
		ier = integer(1))
	if(warn & r$ier == 1)
		print(" estimate nonzero outside ab rectangle")
	list(z = matrix(r$f, nbin[1], nbin[2]), x = center(ab[1,  ], nbin[1]), 
		y = center(ab[2,  ], nbin[2]), m = m, ab = ab, kopt = kopt, ier
		 = r$ier)
}

"bin1"<- function(x, ab = nicerange(x), nbin = 50, warn = T)
{
	n <- length(x)
	if(ab[1] >= ab[2])
		stop("Interval vector has negative orientation")
	if(nbin <= 0)
		stop("Number of bin intervals nonpositive")
	r <- .C("bin1",
		as.single(x),
		as.integer(n),
		as.single(ab),
		as.integer(nbin),
		nc = integer(nbin),
		nskip = integer(1))
	if(warn & r$nskip > 0)
		print(paste(r$nskip, " points outside the interval ab"))
	list(nc = r$nc, ab = ab, nskip = r$nskip)
}

"bin2"<- function(x, ab, nbin = c(30, 30), warn = T)
{
	if(missing(ab)) {
		ab <- t(apply(x, 2, nicerange))
	}
	n <- nrow(x)
	r <- .C("bin2",
		as.single(x),
		as.integer(n),
		as.single(ab),
		as.integer(nbin),
		nc = integer(nbin[1] * nbin[2]),
		nskip = integer(1))
	if(warn & r$nskip > 0)
		print(paste(r$nskip, " points outside the intervals ab"))
	list(nc = matrix(r$nc, nbin[1], nbin[2]), ab = ab, nskip = r$nskip)
}

"center"<- function(ab, k)
{
# compute centers of histogram bins
	if(length(ab) != 2) stop("ab not of length 2")
	h <- diff(ab)/k
	seq(ab[1] + h/2, by = h, length = k)
}

"compress"<- 
function(x, ab = t(apply(x, 2, nicerange)), nbin = rep(30, ncol(x)))
{
	n <- nrow(x)
	nvar <- ncol(x)
	if(ncol(ab) != 2)
		stop("ab should have 2 columns")
	if(nrow(ab) != nvar || length(nbin) != nvar)
		stop("dimensions of x,ab,nbin do not agree")
	if(min(nbin) <= 0) stop("nbin must be positive")	
##### S function to compress data into binned integer form
##### Copyright 1988,1992 David W. Scott
#	x	- input trivariate data matrix  (n x nvar)
#	ab	- input bin range matrix (nvar x 2)
#			x binned over [ ab[1,1], ab[1,2] )
#			y binned over [ ab[2,1], ab[2,2] )
#			z binned over [ ab[3,1], ab[3,2] ) etc.
#	nbin	- vector length nvar - number of bins along each axis
#	nout	- subset of n data points in bin range
#	nc	- output matrix (nout x nvar) containing trivariate bin numbers
	del <- (ab[, 2] - ab[, 1])/nbin	# bin widths
	iskip <- 0	# number of points not in trivariate bins
	nout <- 0	# number of points in output array
# loop on data
	nc <- matrix(0, n, nvar)
	k <- NULL
	for(i in 1:nvar) {
		if(ab[i, 1] >= ab[i, 2])
			stop("ab array incorrectly ordered")
		nc[, i] <- floor((x[, i] - ab[i, 1])/del[i] + 1)
		k <- unique(c(k, seq(n)[nc[, i] < 1 | nc[, i] > nbin[i]]))
	}
	iskip <- length(k)
	if(iskip > 0) {
		nc <- nc[ - k,  ]
		print(paste(as.character(iskip), 
			"point(s) not in hyper-rectangle"))
	}
	storage.mode(nc) <- "integer"
	storage.mode(ab) <- "single"
	storage.mode(nbin) <- "integer"
	return(nc, ab, nbin)
}

"nicerange"<- function(x, beta = 0.1)
{
	ab <- range(x)	# interval surrounding data
	del <- ((ab[2] - ab[1]) * beta)/2
	return(c(ab + c( - del, del)))
}

".First.lib"<-
function(lib.loc, section)
{
        path <- paste(lib.loc, section, "ash.obj", sep = "/")
        dyn.load(path)
}
